/*********************************************************************
 *  SPDX-License-Identifier: MIT                                     *
 *  Copyright (C) 2006-2022 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of qaplay.                                     *
 *********************************************************************/
#ifndef _QAPLAY_H_
#define _QAPLAY_H_

#include <QAction>
#include <QApplication>
#include <QAudioOutput>
#include <QDialog>
#include <QDir>
#include <QFileInfo>
#include <QIcon>
#include <QList>
#include <QMediaMetaData>
#include <QMediaPlayer>
#include <QMenu>
#include <QMessageBox>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QSystemTrayIcon>
#include <QTime>
#include <QUrl>
#include <QVariant>
#include <QWidget>

#include "abouthelp.h"
#include "structs.h"
#include "m3u.h"
//#include "info.h"
#include "editplaylist.h"
#include "setplaylist.h"

#include <iostream>
using namespace std;


class Qaplay : public QSystemTrayIcon
{
	Q_OBJECT

public:
	Qaplay(QSystemTrayIcon *parent = 0);
		~Qaplay();

private:
//	bool bAh;				// about/help widget is shown
	bool bD;

	int iM3uLoaded;			// m3u playlist: -1=undefined after boot, 0=not available 1=writable but not yet loaded, 2=loaded into slM3u
	int iD;
	int iD2;
	int iInvokedBy;			// qaplay was called with -1=unknown, 1=a directory, 2=a song, 3=m3u list
	int iSongCurListIdx;	// index of current song in song list
	int iWShown;			// widget shown: 0=none, 1=AboutHelp/about, 2=AboutHelp/help, 3=SetPlayList, 4=EditPlayList, 5=SongList

	QAction *actAbout;				// show about dialog
	QAction *actHelp;				// show online help
	QAction *actQuit;				// quit program

	QAction *actEditPlayListShow;	// show widget to edit current playlist
	QAction *actEditPlayListDelete;	// hide widget to edit current playlist

	QAction *actPlayListAdd;		// add song to current playlist
	QAction *actPlayListPlay;		// play playlist

	QAction *actSetPlayListShow;	// show widget to set current playlist
	QAction *actSetPlayListDelete;	// delete widget to set current playlist

	QAction *actSongListShow;		// show current song list
	QAction *actSongListDelete;		// delete current song list window

	QAction *actInfo;				// show file/song information
	QAction *actNext;				// play next song
	QAction *actPause;				// pause/resume playing
	QAction *actPrev;				// play previous songg
	QAction *actStop;				// stop playing

	QAudioOutput *audioOutput;
	QMediaMetaData metaData;
	QMediaPlayer *player;

	QMenu *menu;

	QString sD;
	QString sD2;			// for debugin only
	QString sFileInvoked;	// file qaplay was called with - can be song, dir or m3u
	QString sM3u;			// current playlist (to add to)
	QString sPlayListCur;	// name of current playlist
	QString sSongCur;		// current song (to play)
	QString sQtt;			// qtooltip for the tray icon

	QStringList slClArgs;	// program command line arguments
	QStringList slD;
	QStringList slFilters;	// dir filter to only catch song files
	QStringList slM3u;		// current internal m3u playlist - incl. #EXTINF: lines
//	QStringList slPArgs;	// qprocess command arguments
	QStringList slSongs;	// song list to play (in dir), only full path/name, not m3u format

	StrSongInfo strSiD;		// song info struct entity
	
	StrM3u strM3uItemD;			// playlist item struct entity
	QList<StrM3u> liM3uItems;	// list with songs in StrM3u items
	QList<StrM3u> liM3uItemsD;	// list with songs in StrM3u items

	QVariant vD;

	AboutHelp *ahw;			// help widget
	EditPlayList *eplw;		// edit playlist widget
	SetPlayList *splw;		// set playlist widget

private slots:
	void ahAbout();
	void ahHelp();
	void ahShow();
	void ahDelete();

	void quit();

	void actionsDef();
	void menuCreate();

	void playCreate();			// create player
	void playPlay();			// play song of song list
	void playPrep();			// prepare playing
	void playPrev();
	void playNext();
	void playPause();
	void playReadDir();			// read dir, search for files to play
	void playStatusChanged(QMediaPlayer::MediaStatus state);
	void playStop();
	void playInfoShow();		// show song info
	QString playInfoFormatTime(int iD);

	int plCurrentCheckAvail();	// check whether a previous used playlist is available
	void plCurrentLoad();		// load the current m3u
	void plCurrentPlayList();	// play current playlist
	void plCurrentSetup();		// load internal playlist only when first needed - fast path
	void plCurrentSongAdd();	// add current song to current playlist
	void plCurrentUpdated(const QList<StrM3u>& liM3uItemsD);	// update current playlist from edit playlist widget
	bool plCurrentWrite();		// save current favorites playlist
//	void plCurrentSongDel();	// delete selected song from playlist
//	void plCurrentSongRename();	// rename current song in playlist

	void editPlaylistWdgtShow();	// show edit playlist widget
	void editPlaylistWdgtDelete();	// delete edit playlist widget

//	void playListsCheckSupported();	// check if we can support playlists
//	void setPlayListPlDeleted();					// delete a playlist
	void setPlayListPlUpdated(const QString& sD);	// current playlist has been updated - set
	void setPlayListPlNew(const QString& sD);		// create a new, empty playlist
	void setPlayListPlRenamed();					// current playlist has been renamed
	void setPlayListWdgtShow();		// show set playlist widget
	void setPlayListWdgtDelete();	// delete set playlist widget

	void settingsRead();
	void settingsWrite();

	void songListShow();		// show songlist
	void songListDelete();		// delete songlist window

	void toolTipUpd();			// update tooltip with song currently played

	signals:
		void sigQapAh(int iD);				// define which widget to show: about or help

		void sigQapEplShow(const QString& sD, const QList<StrM3u>& sLiM3uItems);	// send current playlist to playlist widget

		void sigQapSplShow(const QString& sD);	// send full path n name of current playlist to set playlist widget
};

#endif	// _QAPLAY_H_
